@@ -6,19 +6,17 @@
 @desc: zyl utils
 """
 
-
-import os
-import time
-
-import pandas as pd
-import wandb
 import re
-from loguru import logger
+
+import langid
+import nltk
+import pandas as pd
+from tqdm import tqdm
+
 
 # from pharm_ai.config import ConfigFilePaths
 
 # from pharm_ai.util.utils import Utilfuncs
-from simpletransformers.classification import ClassificationModel, ClassificationArgs, DDPClassificationModel
 
 
 class DTUtils:
@@ -72,7 +70,7 @@ class DTUtils:
     def cut_train_eval(all_df):
         from sklearn.utils import resample
         raw_df = resample(all_df, replace=False)
-        cut_point = min(8000, int(0.2 * len(raw_df)))
+        cut_point = min(5000, int(0.2 * len(raw_df)))
         eval_df = raw_df[0:cut_point]
         train_df = raw_df[cut_point:]
         return train_df, eval_df
@@ -175,8 +173,109 @@ class DTUtils:
             cmd = 'rm -rf ' + args.output_dir.split('outputs')[0] + 'cache/'
             os.system(cmd)
 
+    @staticmethod
+    def word_piece_tokenizer(text):
+        if langid.classify(str(text))[0] == 'zh':
+            return [i for i in text]
+        else:
+            return nltk.word_tokenize(text)
+
+    @staticmethod
+    def sunday_match(target, pattern):
+        """
+
+        Args:
+            target:
+            pattern:
+
+        Returns:
+
+        """
+        len_target = len(target)
+        len_pattern = len(pattern)
+
+        if len_pattern > len_target:
+            return list()
+
+        index = 0
+        starts = []
+        while index < len_target:
+            if pattern == target[index:index + len_pattern]:
+                starts.append(index)
+                index += 1
+            else:
+                if (index + len(pattern)) >= len_target:
+                    return starts
+                else:
+                    if target[index + len(pattern)] not in pattern:
+                        index += (len_pattern + 1)
+                    else:
+                        index += 1
+        return starts
+
+    @staticmethod
+    def transfomer_data_format_from_t5_to_ner(df: pd.DataFrame, delimiter='|',
+                                              keep_addition_info=('id','text_type')):
+        """
+
+        Args:
+            df: dataframe,must have the columns-['prefix','input_text','target_text']
+
+        Returns:
+
+        """
+        all_cls = df.value_counts('prefix').index.to_list()
+        custom_labels = ['O']
+        for c in all_cls:
+            custom_labels.append('B-' + c.upper())
+            custom_labels.append('I-' + c.upper())
+        sentence_id = 0
+        res_li = []
+
+        for input_text, sub_df in tqdm(df.groupby('input_text',sort=False)):
+            words = DTUtils.word_piece_tokenizer(input_text)
+            labels = ['O'] * len(words)
+
+            for _, d in sub_df.iterrows():
+                if keep_addition_info:
+                    for k in range(len(keep_addition_info)):
+                        exec(f'info_{k} = d[keep_addition_info[{k}]]')
+
+                cls = d['prefix']
+                sub_label = set(d['target_text'].split(delimiter))
+                while '' in sub_label:
+                    sub_label.remove('')
+                if sub_label:
+                    for every_entity in sub_label:
+                        entity = DTUtils.word_piece_tokenizer(every_entity)
+                        res_starts = DTUtils.sunday_match(target=words,pattern=entity)
+                        if res_starts:
+                            for r in res_starts:
+                                labels[r] = 'B-' + cls.upper()
+                                if len(entity)>1:
+                                    labels[r + 1: r + len(entity)] = ['I-' + cls.upper()] * (len(entity) - 1)
+
+            sentence_ner = []
+            for w,l in zip(words,labels):
+                r = {'sentence_id': sentence_id, 'words': w, 'labels': l}
+                if keep_addition_info:
+                    for k in range(len(keep_addition_info)):
+                        r.update({keep_addition_info[k]:eval(f'info_{k}')})
+                sentence_ner.append(r)
+
+            res_li.extend(sentence_ner)
+            sentence_id += 1
+
+        df = pd.DataFrame(res_li)
+
+        return df
+
+
 
 if __name__ == '__main__':
+    test_df = pd.read_excel("/home/zyl/disk/PharmAI/pharm_ai/panel/data/v2.4.c/processed_0820.xlsx", 'eval')[0:100]
+
+    DTUtils.transfomer_data_format_from_t5_to_ner(test_df)
     pass
     # class Project(MyModel):
     #     def __init__(self):
@@ -219,8 +318,8 @@ if __name__ == '__main__':
     # # num = 5
     # # print(DTUtils.split_data_evenly(d, 5))
     # # print('1')
-    # r = 'ssdfsdfsaas dsdfs dfsdf'
-    # f = 'sd'
+    # r = ['a',' ','','df','x',]
+    # f = ['','df']
     # g = DTUtils.find_index(r, f)
     # print(g)
     # for i in g:
